# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import exceptions, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin

from ... import models
from . import serializers

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# ADDRESS SCOPE
class AddressScopeViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    AddressScopeViewSet
    -----------

    :action list: Return the address scope list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given address scope.
        `Profiles`: `superuser`, `staff`

    :action light: Return the address scope list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new address scope instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a address scope instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a address scope instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a address scope instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.AddressScope.objects.all()
    serializer_class = serializers.AddressScopeSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'AddressScope{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a address scope instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a address scope instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ADDRESS
class AddressViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    AddressViewSet
    -----------

    :action list: Return the address list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given address.
        `Profiles`: `superuser`, `staff`

    :action light: Return the address list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new address instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a address instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a address instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a address instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Address.objects.all()
    serializer_class = serializers.AddressSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Address{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a address instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a address instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# ADDRESS COORDINATE
class AddressCoordinateViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    AddressCoordinateViewSet
    -----------

    :action list: Return the address coordinate list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given address coordinate.
        `Profiles`: `superuser`, `staff`

    :action light: Return the address coordinate list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new address coordinate instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a address coordinate instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a address coordinate instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a address coordinate instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.AddressCoordinate.objects.all()
    serializer_class = serializers.AddressCoordinateSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'AddressCoordinate{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a address coordinate instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a address coordinate instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
